@@ -1,7 +1,7 @@
 /*!
 # Execute
 
-This library is for extending `Command` in order to execute programs more easily.
+This library is used for extending `Command` in order to execute programs more easily.
 
 ## Usage
 
@@ -206,7 +206,7 @@ println!("{}", String::from_utf8(output.stdout).unwrap());
 # }
 ```
 
-### Execute Many Commands and Pipe Them Together
+### Execute Multiple Commands and Pipe Them Together
 
 ```rust
 extern crate execute;
@@ -227,17 +227,64 @@ command3.arg("A-Z").arg("a-z");
 
 command3.stdout(Stdio::piped());
 
-let output = command1.execute_many_output(&mut [&mut command2, &mut command3]).unwrap();
+let output = command1.execute_multiple_output(&mut [&mut command2, &mut command3]).unwrap();
 
 assert_eq!(b"hello\n", output.stdout.as_slice());
 # }
 ```
+
+### Run a Command String in the Current Shell
+
+The `shell` function can be used to create a `Command` instance with a single command string instead of a program name and scattered arguments.
+
+```rust
+extern crate execute;
+
+use std::process::{Command, Stdio};
+
+use execute::{Execute, shell};
+
+# if cfg!(target_os = "linux") {
+let mut command = shell("cat /proc/meminfo");
+
+command.stdout(Stdio::piped());
+
+let output = command.execute_output().unwrap();
+
+println!("{}", String::from_utf8(output.stdout).unwrap());
+# }
+```
+
+### Parse a Command String
+
+The `command` function can be used to create a `Command` instance with a single command string instead of a program name and scattered arguments. The difference between the `shell` function and the `command` function is that the former is interpreted by the current shell while the latter is parsed by this crate.
+
+```rust
+extern crate execute;
+
+use std::process::{Command, Stdio};
+
+use execute::{Execute, command};
+
+# if cfg!(target_os = "linux") {
+let mut command = command("cat '/proc/meminfo'");
+
+command.stdout(Stdio::piped());
+
+let output = command.execute_output().unwrap();
+
+println!("{}", String::from_utf8(output.stdout).unwrap());
+# }
+```
 */
 
 pub extern crate generic_array;
 
+use std::env;
+use std::ffi::{OsStr, OsString};
 use std::io::{self, ErrorKind, Read, Write};
 use std::process::{Command, Output, Stdio};
+use std::sync::Once;
 
 use generic_array::typenum::{IsGreaterOrEqual, True, U1, U256};
 use generic_array::{ArrayLength, GenericArray};
@@ -297,23 +344,24 @@ pub trait Execute {
         reader: &mut dyn Read,
     ) -> Result<Output, io::Error>;
 
-    /// TODO execute_many
+    /// TODO execute_multiple
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and get the exit status code. The stdin of the first process will be set to `Stdio::inherit()`. The stdout and stderr of the last process will be set to `Stdio::null()`.
-    fn execute_many(&mut self, others: &mut [&mut Command]) -> Result<Option<i32>, io::Error>;
+    fn execute_multiple(&mut self, others: &mut [&mut Command]) -> Result<Option<i32>, io::Error>;
 
     /// Execute this command as well as other commands and pipe their stdin and stdout. The stdin of the first process will be set to `Stdio::inherit()`. By default, the stdout and stderr of the last process are inherited from the parent.
-    fn execute_many_output(&mut self, others: &mut [&mut Command]) -> Result<Output, io::Error>;
+    fn execute_multiple_output(&mut self, others: &mut [&mut Command])
+        -> Result<Output, io::Error>;
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and input in-memory data to the process, and get the exit status code. The stdin of the first process will be set to `Stdio::piped()`. The stdout and stderr of the last process will be set to `Stdio::null()`.
-    fn execute_many_input<D: ?Sized + AsRef<[u8]>>(
+    fn execute_multiple_input<D: ?Sized + AsRef<[u8]>>(
         &mut self,
         data: &D,
         others: &mut [&mut Command],
     ) -> Result<Option<i32>, io::Error>;
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and input in-memory data to the process. The stdin of the first process will be set to `Stdio::piped()`. By default, the stdout and stderr of the last process are inherited from the parent.
-    fn execute_many_input_output<D: ?Sized + AsRef<[u8]>>(
+    fn execute_multiple_input_output<D: ?Sized + AsRef<[u8]>>(
         &mut self,
         data: &D,
         others: &mut [&mut Command],
@@ -321,16 +369,16 @@ pub trait Execute {
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and input data from a reader to the process, and get the exit status code. The stdin of the first process will be set to `Stdio::piped()`. The stdout and stderr of the last process will be set to `Stdio::null()`.
     #[inline]
-    fn execute_many_input_reader(
+    fn execute_multiple_input_reader(
         &mut self,
         reader: &mut dyn Read,
         others: &mut [&mut Command],
     ) -> Result<Option<i32>, io::Error> {
-        self.execute_many_input_reader2::<U256>(reader, others)
+        self.execute_multiple_input_reader2::<U256>(reader, others)
     }
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and input data from a reader to the process, and get the exit status code. The stdin of the first process will be set to `Stdio::piped()`. The stdout and stderr of the last process will be set to `Stdio::null()`.
-    fn execute_many_input_reader2<N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>>(
+    fn execute_multiple_input_reader2<N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>>(
         &mut self,
         reader: &mut dyn Read,
         others: &mut [&mut Command],
@@ -338,16 +386,18 @@ pub trait Execute {
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and input data from a reader to the process. The stdin of the first process will be set to `Stdio::piped()`. By default, the stdout and stderr of the last process are inherited from the parent.
     #[inline]
-    fn execute_many_input_reader_output(
+    fn execute_multiple_input_reader_output(
         &mut self,
         reader: &mut dyn Read,
         others: &mut [&mut Command],
     ) -> Result<Output, io::Error> {
-        self.execute_many_input_reader_output2::<U256>(reader, others)
+        self.execute_multiple_input_reader_output2::<U256>(reader, others)
     }
 
     /// Execute this command as well as other commands and pipe their stdin and stdout, and input data from a reader to the process. The stdin of the first process will be set to `Stdio::piped()`. By default, the stdout and stderr of the last process are inherited from the parent.
-    fn execute_many_input_reader_output2<N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>>(
+    fn execute_multiple_input_reader_output2<
+        N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>,
+    >(
         &mut self,
         reader: &mut dyn Read,
         others: &mut [&mut Command],
@@ -457,7 +507,7 @@ impl Execute for Command {
         child.wait_with_output()
     }
 
-    fn execute_many(&mut self, others: &mut [&mut Command]) -> Result<Option<i32>, io::Error> {
+    fn execute_multiple(&mut self, others: &mut [&mut Command]) -> Result<Option<i32>, io::Error> {
         if others.is_empty() {
             return self.execute();
         }
@@ -487,7 +537,10 @@ impl Execute for Command {
         Ok(last_other.status()?.code())
     }
 
-    fn execute_many_output(&mut self, others: &mut [&mut Command]) -> Result<Output, io::Error> {
+    fn execute_multiple_output(
+        &mut self,
+        others: &mut [&mut Command],
+    ) -> Result<Output, io::Error> {
         if others.is_empty() {
             return self.execute_output();
         }
@@ -515,7 +568,7 @@ impl Execute for Command {
         last_other.spawn()?.wait_with_output()
     }
 
-    fn execute_many_input<D: ?Sized + AsRef<[u8]>>(
+    fn execute_multiple_input<D: ?Sized + AsRef<[u8]>>(
         &mut self,
         data: &D,
         others: &mut [&mut Command],
@@ -551,7 +604,7 @@ impl Execute for Command {
         Ok(last_other.status()?.code())
     }
 
-    fn execute_many_input_output<D: ?Sized + AsRef<[u8]>>(
+    fn execute_multiple_input_output<D: ?Sized + AsRef<[u8]>>(
         &mut self,
         data: &D,
         others: &mut [&mut Command],
@@ -585,7 +638,7 @@ impl Execute for Command {
         last_other.spawn()?.wait_with_output()
     }
 
-    fn execute_many_input_reader2<N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>>(
+    fn execute_multiple_input_reader2<N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>>(
         &mut self,
         reader: &mut dyn Read,
         others: &mut [&mut Command],
@@ -634,7 +687,7 @@ impl Execute for Command {
         Ok(last_other.status()?.code())
     }
 
-    fn execute_many_input_reader_output2<
+    fn execute_multiple_input_reader_output2<
         N: ArrayLength<u8> + IsGreaterOrEqual<U1, Output = True>,
     >(
         &mut self,
@@ -683,3 +736,121 @@ impl Execute for Command {
         last_other.spawn()?.wait_with_output()
     }
 }
+
+/// Create a `Command` instance which can be executed by the current command language interpreter (shell).
+#[cfg(unix)]
+pub fn shell<S: AsRef<OsStr>>(cmd: S) -> Command {
+    static START: Once = Once::new();
+    static mut SHELL: Option<OsString> = None;
+
+    let shell = unsafe {
+        START.call_once(|| {
+            SHELL = Some(env::var_os("SHELL").unwrap_or_else(|| OsString::from(String::from("sh"))))
+        });
+
+        SHELL.as_ref().unwrap()
+    };
+
+    let mut command = Command::new(shell);
+
+    command.arg("-c");
+    command.arg(cmd);
+
+    command
+}
+
+/// Create a `Command` instance which can be executed by the current command language interpreter (shell).
+#[cfg(windows)]
+pub fn shell<S: AsRef<OsStr>>(cmd: S) -> Command {
+    let mut command = Command::new("cmd.exe");
+
+    command.arg("/c");
+    command.arg(cmd);
+
+    command
+}
+
+/// Create a `Command` instance by parsing a command string.
+pub fn command<S: AsRef<str>>(cmd: S) -> Command {
+    let cmd = cmd.as_ref();
+
+    let mut tokens = Vec::with_capacity(1);
+    let mut string_buffer = String::new();
+
+    let mut append_mode = false;
+    let mut quote_mode = false;
+    let mut quote_mode_ending = false; // to deal with '123''456' -> 123456
+    let mut quote_char = ' ';
+    let mut escaping = false;
+
+    for c in cmd.chars() {
+        if escaping {
+            append_mode = true;
+            escaping = false;
+
+            string_buffer.push(c);
+        } else if c.is_whitespace() {
+            if append_mode {
+                if quote_mode {
+                    string_buffer.push(c);
+                } else {
+                    append_mode = false;
+
+                    tokens.push(string_buffer);
+                    string_buffer = String::new();
+                }
+            } else if quote_mode_ending {
+                quote_mode_ending = false;
+
+                tokens.push(string_buffer);
+                string_buffer = String::new();
+            }
+        } else {
+            match c {
+                '"' | '\'' => {
+                    if append_mode {
+                        if quote_mode {
+                            if quote_char == c {
+                                append_mode = false;
+                                quote_mode = false;
+                                quote_mode_ending = true;
+                            } else {
+                                string_buffer.push(c);
+                            }
+                        } else {
+                            quote_mode = true;
+                            quote_char = c;
+                        }
+                    } else {
+                        append_mode = true;
+                        quote_mode = true;
+                        quote_char = c;
+                    }
+                }
+                '\\' => {
+                    escaping = true;
+                }
+                _ => {
+                    append_mode = true;
+                    escaping = false;
+
+                    string_buffer.push(c);
+                }
+            }
+        }
+    }
+
+    if append_mode || quote_mode_ending {
+        tokens.push(string_buffer);
+    }
+
+    if tokens.is_empty() {
+        Command::new("")
+    } else {
+        let mut command = Command::new(&tokens[0]);
+
+        command.args(&tokens[1..]);
+
+        command
+    }
+}
